<?php

namespace Drupal\node_by_term\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\node_by_term\Form\NodeByTermForm;
use Drupal\Core\Database\Connection;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Url;
use Drupal\Component\Render\FormattableMarkup;
use Drupal\Core\Link;
use Drupal\Core\Pager\PagerManagerInterface;
use Symfony\Component\HttpFoundation\Request;
use Drupal\node\Entity\Node; 

/**
 * NodeListController.
 */

class NodeListController extends ControllerBase {
  /**
   * {@inheritdoc}
   */
  public function nodelist() {
    $data['filter'] = \Drupal::formBuilder()->getForm('Drupal\node_by_term\Form\NodeByTermForm');
    $vocab = \Drupal::request()->query->get('vocab');
    $t_id = \Drupal::request()->query->get('t_id');
    $content_type = \Drupal::request()->query->get('cont_type');
          
    // Load the vocabulary by its machine name.
    if(!empty($vocab)){
      $vocabulary = \Drupal\taxonomy\Entity\Vocabulary::load($vocab);
      $vocabulary_id = $vocabulary->id();
      $term_ids = \Drupal::entityQuery('taxonomy_term')
          ->condition('vid', $vocabulary_id)
          ->accessCheck(FALSE)  // This line is added.
          ->execute();
    }

    if(!empty($vocab) AND  empty($t_id) AND empty($content_type)) {
        $query = \Drupal::database()->select('taxonomy_index', 'ti');
        $query->join('node_field_data', 'nfd', 'nfd.nid = ti.nid');
        $query->fields('nfd', ['created', 'changed']);
        $query->fields('ti', ['nid']);
        $query->condition('ti.tid', $term_ids, 'IN');
          
        //For the pagination we need to extend the pagerselectextender and
        $pager = $query->extend('Drupal\Core\Database\Query\PagerSelectExtender')->limit(2);
        $nids = $pager->execute()->fetchall();
        
        $nid = [];
        foreach($nids as $ids) {
          $nid[] = $ids->nid;
        }
        $nodes = Node::loadMultiple($nid);
    }
    elseif(!empty($vocab AND $t_id)) {
      $query = \Drupal::database()->select('taxonomy_index', 'ti');
      $query->join('node_field_data', 'nfd', 'nfd.nid = ti.nid');
      $query->fields('nfd', ['created', 'changed']);
      $query->fields('ti', ['nid']);
      $query->condition('ti.tid', $t_id);
      // Add the condition on the content type, if content type is set.
        if (isset($content_type) && !empty($content_type)) {
          $query->condition('nfd.type', $content_type);
        }
      //For the pagination we need to extend the pagerselectextender and
      $pager = $query->extend('Drupal\Core\Database\Query\PagerSelectExtender')->limit(2);
      $nids = $pager->execute()->fetchall();
      $nid = [];
      foreach($nids as $ids) {
        $nid[] = $ids->nid;
      }
      $nodes = Node::loadMultiple($nid);
    } 
    else{
      $query = \Drupal::database()->select('node_field_data', 'nfd');
      $query->fields('nfd', ['nid']);
      $query->addField('nfd', 'created');
      $query->addField('nfd', 'changed');
      if(isset($content_type) && !empty($content_type))
      {
        $query->condition('nfd.type', $content_type);
      }
      $pager = $query->extend('Drupal\Core\Database\Query\PagerSelectExtender')->limit(2);       
      $nids = $pager->execute()->fetchCol();                          
      $nid = [];
      foreach($nids as $ids) {
        $nid[] = $ids;
      }
      $nodes = Node::loadMultiple($nid);
    }

  
    // $nodes = Node::loadMultiple($nid);

    // Prepare the header for the table.
    $header = [
      'title' => $this->t('Node Title'),
      'content_type' => $this->t('Content Type'),
      'status' => $this->t('Published Status'),
      'created-date'=> $this->t('Created date'),
      'last modified date'=> $this->t('Last modified'),
      'Operations' => $this->t('Operations'),
    ];
    // Prepare the rows for the table.
    $rows = [];

    foreach ($nodes as $node) 
    {
      // Format dates.
      $created_date = \Drupal::service('date.formatter')->format($node->getCreatedTime(), 'custom', 'd-m-Y H:i:s');
      $changed_date = \Drupal::service('date.formatter')->format($node->getChangedTime(), 'custom', 'd-m-Y H:i:s');
     
      
      $edit_url = Url::fromRoute('entity.node.edit_form', ['node' => $node->id()])->toString();
      $edit_link = new FormattableMarkup('<a href=":link">:label</a>', [':link' => $edit_url, ':label' => $this->t('Edit')]);
      // Create "View" link.
      $view_url = Url::fromRoute('entity.node.canonical', ['node' => $node->id()]);
      $view_link = Link::fromTextAndUrl($this->t('View'), $view_url)->toString();
      // Render the links.
      $links = new FormattableMarkup('@edit_link | @view_link', ['@edit_link' => $edit_link, '@view_link' => $view_link]);
     
      $rows[] = [
        'title' => $node->getTitle(),
        'content_type' => $node->getType(),
        'status' => $node->isPublished() ? $this->t('Published') : $this->t('Unpublished'),
        'created-date' => $created_date,
        'last modified date' => $changed_date,
        'edit' => $links,
      ];
    }

    // Build the table render array.
    $data['table'] = [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No nodes founds.'),
    ];
    $data['pager'] = [
      '#type' => 'pager'
    ];
    // Prevent caching.
    $data['#cache']['max-age'] = 0;

    return $data;
  }
}