<?php

namespace Drupal\node_by_term\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\taxonomy\Entity\Term;

/**
 * Provides a form to filter nodes.
 */
class NodeByTermForm extends FormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'node_by_term_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    //get elements from query parameter 

    $vocab = \Drupal::request()->query->get('vocab');
    $t_id = \Drupal::request()->query->get('t_id');
    $content_type_value = \Drupal::request()->query->get('cont_type');
    
    //this will load all vocabulary
    $vocabularies = Vocabulary::loadMultiple();
    $options = ['-select-'];
    foreach ($vocabularies as $vocabulary) {
      $options[$vocabulary->id()] = $vocabulary->label();
    }

    $form['vocabulary'] = [
      '#type' => 'select',
      '#title' => $this->t('Vocabulary'),
      '#options' => $options,
      '#default_value'=> !empty($vocab) ? $vocab : '', 
      '#ajax' => [
        'callback' => '::load_term',
        'event' => 'change',
        'wrapper' => 'term-wrapper',
      ], 
      '#attributes' => [
        'class' => ['form-control'],
      ],     
    ];

    // If vocabulary is selected, load its terms.
    $selected_vocabulary = $form_state->getValue('vocabulary');

    if (!empty($selected_vocabulary)) {
      $terms = \Drupal::entityTypeManager()
        ->getStorage('taxonomy_term')
        ->loadTree($selected_vocabulary);
      $term_options = ['-select-'];
      foreach ($terms as $term) {
        $term_options[$term->tid] = $term->name;
      }
    }
    else {
      // this condition will apply after submitting
      $terms = \Drupal::entityTypeManager()
      ->getStorage('taxonomy_term')
      ->loadTree($vocab);

      $term_options = ['-select-'];
      foreach ($terms as $term) {
        $term_options[$term->tid] = $term->name;
      }
    }

    $form['term'] = [
      '#type' => 'select',
      '#title' => $this->t('Term'),
      '#prefix' => '<div id="term-wrapper"  >',
      '#suffix' => '</div>',
      '#options' => !empty($term_options) ? $term_options : ['No term'] ,
      '#default_value' =>!empty($t_id) ? $t_id :  ' ',
      '#attributes' => [
        'class' => ['form-control'],
      ],
    ];

    // Fetch all content types.
    $content_types = \Drupal::service('entity_type.manager')->getStorage('node_type')->loadMultiple();
    $content_type_options = ['-select-'];
    
    foreach ($content_types as $content_type) {
      $content_type_options[$content_type->id()] = $content_type->label();
    }

    $form['content_type'] = [
      '#type' => 'select',
      '#title' => $this->t('Content Type'),
      '#options' => $content_type_options,
      '#default_value' => $content_type_value,
      '#attributes' => [
        'class' => ['form-control'],
      ],
    ];

    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Submit'),
      '#attributes' => [
        'class' => ['btn btn-success'],
      ],      
    ];
   
    $form['actions']['reset'] = array(
      '#type' => 'submit',
      '#value' => $this->t('Reset'),
      '#submit' => array('::resetForm'),
      '#attributes' => [
        'class' => ['btn btn-danger'],
      ],
    );
    $form['#attached']['library'][] = 'node_by_term/my_styles';


    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function load_term(array $form, FormStateInterface $form_state) {
    return $form['term'];
  }
  public function resetForm(array &$form, FormStateInterface $form_state) {
  $form_state->setRedirect('node_by_term.nodelist'); 
}


  public function submitForm(array &$form, FormStateInterface $form_state) {
    $vocab = $form_state->getValue('vocabulary');  
    $t_id = $form_state->getValue('term');   
    $cont_type = $form_state->getValue('content_type');
    
    $url = \Drupal\Core\Url::fromRoute('node_by_term.nodelist')->setRouteParameters(['vocab' => $vocab, 't_id' => $t_id , 'cont_type' => $cont_type]);
    $form_state->setRedirectUrl($url);   
  }
}